/*******************************************************************************

 stratego
 
 Visor para las partidas de stratego realizadas con CLIPS y los ficheros
 de la clase de Diseño de Videojuegos.
 
 
 Por Roberto García Carvajal.

 Disponible bajo los términos de la GNU General Public License (GPL) version 2
o superior

*******************************************************************************/

#include <SDL/SDL.h>
#include <fstream>
#include <iostream>

using namespace std;

#define _EQUIPO 0
#define _RANGO  1
#define _X      2
#define _Y      3
#define _DESC   4

// Función que dibuja una escena de la partida.
void dibujar(SDL_Surface * pantalla,SDL_Surface ** imagenes,unsigned char ** ronda);

int main(int argc, char *argv[])
{
 if (argc < 2)
 {
 	cout << "Error, el programa necesita un fichero resultado para comenzar la partida." << endl;
 	exit(1);
 }

 // Estructura que contendrá todas las rondas. Ocupa unos 20.004 bytes, por lo
 // que no supone un derroche de memoria.
 unsigned char ***rondas;
 rondas = new unsigned char**[200];
 
 // Dejo toda la estructura llena de ceros.
 for(unsigned int i = 0;i < 200;i++)
 {
  rondas[i] = new unsigned char*[8];
  for(unsigned int j = 0;j < 8;j++)
  {
   rondas[i][j] = new unsigned char[8];
   for(unsigned int k = 0;k < 8;k++)
    rondas[i][j][k] = 0;
  }
 }

 // Inicializo SDL.
 
 atexit(SDL_Quit);

 if (SDL_Init(SDL_INIT_VIDEO) < 0)
 {
  cout << "Imposible inicializar SDL: " << SDL_GetError() << endl;
  exit(1);
 }

 // Creo la pantalla.
 SDL_Surface * pantalla = SDL_SetVideoMode(600,600,24,SDL_HWSURFACE);
 if (pantalla == NULL) 
 {
	 cout << "Imposible crear superficie de pantalla: " << SDL_GetError() << endl;
 	exit(1);
 }

 // Abro el fichero que se pasa como parámetro.
 
 ifstream fich(argv[1],ifstream::in);
 if(!fich)
 {
  cout << "Imposible abrir fichero: \"" << argv[1] << "\"" << endl;
  exit(1);
 }
 // c contiene un caracter temporal.
 unsigned char c;
 // total_turnos contiene el número de turnos en total (y sirve de contador
 // mientras estos se cargan).
 unsigned int total_turnos = 0;
 // Ignoro las dos primeras líneas.
 fich.ignore(10,'\n');
 fich.ignore(10,'\n');
 
 int temporal[5];
 
 while(!fich.eof())
 {
  fich >> c;
  // Si leo una t, significa que estoy en una línea que pone tiempo. 
  // Ignoro esta línea y la siguiente y aumento en 1 el contador total_turnos.
  if(c == 't')
  {
   fich.ignore(10,'\n');
   fich.ignore(10,'\n');
   total_turnos++;
  }
  // Si leo un espacio, significa que se está describiendo el estado de una
  // unidad. La cargo y modifico la estructura.
  else if(c == 'e')
  {
   fich.ignore(10,':');
   fich >> c;
   if(c == 'A')
    temporal[_EQUIPO] = 0;
   else
    temporal[_EQUIPO] = 1;
   
   fich.ignore(10,':');
   fich.ignore(10,':');
   
   fich >> temporal[_RANGO];
   
   fich.ignore(10,':');
   
   fich >> temporal[_X];
   
   fich.ignore(10,':');
   
   fich >> temporal[_Y];
   
   fich.ignore(10,':');
   
   fich >> temporal[_DESC];
   
   fich.ignore(10,'\n');
   
   if(temporal[_DESC])
   {
    if(temporal[_EQUIPO])
     rondas[total_turnos][temporal[_Y]-1][temporal[_X]-1] = temporal[_RANGO] + 6;
    else
     rondas[total_turnos][temporal[_Y]-1][temporal[_X]-1] = temporal[_RANGO];
   }
   else
   {
    if(temporal[_EQUIPO])
     rondas[total_turnos][temporal[_Y]-1][temporal[_X]-1] = temporal[_RANGO] + 18;
    else
     rondas[total_turnos][temporal[_Y]-1][temporal[_X]-1] = temporal[_RANGO] + 12;
   }
  }
 }
 
 // Imágenes de las casillas.
 SDL_Surface * imagenes[25];
 imagenes[0] = SDL_LoadBMP("./imagenes/casilla.bmp");
 imagenes[1] = SDL_LoadBMP("./imagenes/equipoA1.bmp");
 imagenes[2] = SDL_LoadBMP("./imagenes/equipoA2.bmp");
 imagenes[3] = SDL_LoadBMP("./imagenes/equipoA3.bmp");
 imagenes[4] = SDL_LoadBMP("./imagenes/equipoA4.bmp");
 imagenes[5] = SDL_LoadBMP("./imagenes/equipoA5.bmp");
 imagenes[6] = SDL_LoadBMP("./imagenes/equipoA6.bmp");
 imagenes[7] = SDL_LoadBMP("./imagenes/equipoB1.bmp");
 imagenes[8] = SDL_LoadBMP("./imagenes/equipoB2.bmp");
 imagenes[9] = SDL_LoadBMP("./imagenes/equipoB3.bmp");
 imagenes[10] = SDL_LoadBMP("./imagenes/equipoB4.bmp");
 imagenes[11] = SDL_LoadBMP("./imagenes/equipoB5.bmp");
 imagenes[12] = SDL_LoadBMP("./imagenes/equipoB6.bmp");
 imagenes[13] = SDL_LoadBMP("./imagenes/equipoA-1.bmp");
 imagenes[14] = SDL_LoadBMP("./imagenes/equipoA-2.bmp");
 imagenes[15] = SDL_LoadBMP("./imagenes/equipoA-3.bmp");
 imagenes[16] = SDL_LoadBMP("./imagenes/equipoA-4.bmp");
 imagenes[17] = SDL_LoadBMP("./imagenes/equipoA-5.bmp");
 imagenes[18] = SDL_LoadBMP("./imagenes/equipoA-6.bmp");
 imagenes[19] = SDL_LoadBMP("./imagenes/equipoB-1.bmp");
 imagenes[20] = SDL_LoadBMP("./imagenes/equipoB-2.bmp");
 imagenes[21] = SDL_LoadBMP("./imagenes/equipoB-3.bmp");
 imagenes[22] = SDL_LoadBMP("./imagenes/equipoB-4.bmp");
 imagenes[23] = SDL_LoadBMP("./imagenes/equipoB-5.bmp");
 imagenes[24] = SDL_LoadBMP("./imagenes/equipoB-6.bmp");
 // turno_actual contiene el turno que se está mostrando.
 unsigned int turno_actual = 0;
 // continuar indica si se ha de parar la aplicación.
 bool continuar = true;
 // evento para recoger los eventos de teclado.
 SDL_Event evento;
 
 // Dibujo la pantalla.
 dibujar(pantalla,imagenes,rondas[turno_actual]);

 // Bucle principal.
 while(continuar)
 { // Recojo eventos...
  while (SDL_PollEvent(&evento))
  {
   if ( evento.type == SDL_KEYDOWN )
   { // Si se pulsa flecha izquierda, se vuelve al turno anterior.
    if(evento.key.keysym.sym == SDLK_LEFT)
    {
     if(turno_actual)
     {
      turno_actual--;
      dibujar(pantalla,imagenes,rondas[turno_actual]);
     }
    } // Si se pulsa flecha derecha, se pasa al turno siguiente.
    else if(evento.key.keysym.sym == SDLK_RIGHT)
    {
     if(turno_actual !=  total_turnos)
     {
      turno_actual++;
      dibujar(pantalla,imagenes,rondas[turno_actual]);
     }
    } // Si se pulsa escape, se termina.
    else if(evento.key.keysym.sym == SDLK_ESCAPE)
     continuar = false;
   } // Si el gestor de ventanas ordena cerrar la ventana, se termina.
   else if( evento.type == SDL_QUIT )
    continuar = false;
   else if( evento.type == SDL_VIDEOEXPOSE ) // Se redibuja si se mueve la ventana.
    dibujar(pantalla,imagenes,rondas[turno_actual]);
  }
 }

 return 0;
}

void dibujar(SDL_Surface * pantalla,SDL_Surface ** imagenes,unsigned char ** ronda)
{
 SDL_Rect destino;
 destino.x = 0;
 destino.y = 0;
 destino.h = 75;
 destino.w = 75;
 
 for(unsigned int j = 0;j < 8;j++)
 {
  for(unsigned int i = 0;i < 8;i++)
  {
   SDL_BlitSurface(imagenes[ronda[j][i]],NULL,pantalla,&destino);
   destino.x += 75;
  }
  destino.x = 0;
  destino.y += 75;
 }
 SDL_Flip(pantalla);
}
